/*
 * Copyright 2012 David B. Lowery
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.aztechengine.android;

import org.aztechengine.Screen;
import org.aztechengine.State;
import org.aztechengine.android.graphics.Graphics;
import org.aztechengine.android.touch.TouchHandler;

import android.app.Activity;
import android.content.Context;
import android.os.Bundle;
import android.os.PowerManager;
import android.os.PowerManager.WakeLock;
import android.view.Window;
import android.view.WindowManager;

/**
 * An Android game written using AztechEngine only executes one activity. Extend this class
 * (instead of {@link Activity}) in your own Android project to create a game with AztechEngine.
 * 
 * @author lowery
 *
 */
public class AndroidGame extends Activity {
    private static WakeLock wakeLock;
    private static GameSurface surface;
    
    // Game global state variables
    public static int fps;
    
    protected static boolean running;
    protected static Screen screen;
    protected static State state;

    /** 
     * Called when the activity is first created. Initializes the activity with a full wake lock
     * full screen view and no title displayed. 
     * 
     */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);

        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, 
                WindowManager.LayoutParams.FLAG_FULLSCREEN);

        PowerManager powerManager = (PowerManager) getSystemService(Context.POWER_SERVICE);
        wakeLock = powerManager.newWakeLock(PowerManager.FULL_WAKE_LOCK, "FullWakeLock");
        wakeLock.acquire();
        
        surface = new GameSurface(this);
        new TouchHandler(surface); // instance of TouchHandler (contains static members only)
        new Graphics(getAssets());
        
        setContentView(surface);
    }
    
    /**
     * Releases the wake lock on pause.
     * 
     */
    @Override
    public void onPause() {
        super.onPause();
        wakeLock.release();
        surface.stop();
    }
    
    /**
     * Acquires the wake lock on resume.
     * 
     */
    @Override
    public void onResume() {
        super.onResume();
        wakeLock.acquire();
        surface.start();
    }
    
    /**
     * Used to set the {@link Screen} the game should use during the render operations.
     * 
     * @param screen the current Screen object
     * 
     */
    public static synchronized void  setScreen(Screen screen) {
        AndroidGame.screen = screen;
    }
    
    /**
     * Used to set the {@link State} object the game will use for game logic.
     * 
     * @param state the currect State object
     */
    public synchronized void setState(State state) {
        AndroidGame.state = state;
    }
}
