/*
 * Copyright 2012 David B. Lowery
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.aztechengine.android;

import android.content.Context;
import android.graphics.Canvas;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

/**
 * The surface upon which all game screens are drawn. There is only one GameSurface per
 * game created in {@link AndroidGame#onCreate}. Extends an Android
 * {@link SurfaceView}. This class also contains the game loop that runs in a separate
 * thread.
 * 
 * @author lowery
 *
 */
public class GameSurface extends SurfaceView implements Runnable {
    private static final int PERIOD = 20; // one frame every 20ms = 50fps
    
    private Thread thread;
    private static SurfaceHolder surfaceHolder;
    
    /**
     * Constructor obtains the surface holder and uses {@link SurfaceView#setFocusable}
     * to allow the surface view to handle touch events.
     *  
     * @param context the Android Context, specifically an {@link AndroidGame}
     * 
     */
    public GameSurface(Context context) {
        super(context);
        
        surfaceHolder = getHolder();
        setFocusable(true);
    }
    
    /* (non-Javadoc)
     * @see java.lang.Runnable#run()
     */
    public void run() {
        Canvas canvas;
        long timeDiff, timeSleep;
        long timeTotal = 0;
        int frames = 0;
        
        timeDiff = System.currentTimeMillis(); // time before render/update
        
        while(AndroidGame.running) {
            if (surfaceHolder.getSurface().isValid()) {
                canvas = null;
                
                try {
                    canvas = surfaceHolder.lockCanvas();
                    synchronized (surfaceHolder) {
                        update();
                        render(canvas);
                    }
                    timeDiff = System.currentTimeMillis() - timeDiff; // time difference between before and after render/update
                    timeSleep = PERIOD - timeDiff;   // time left in this loop
                    if (timeSleep <= 0)  // update/render took longer than period
                        timeSleep = 5;    // sleep a bit anyway
                    
                    timeTotal += (timeSleep + timeDiff);
                    
                    if (timeTotal > 1000) {
                        AndroidGame.fps = frames;
                        frames = 0;
                        timeTotal = timeTotal % 1000; // carry remaining time over
                    }
                    
                    frames++;
                    Thread.sleep(timeSleep);
                } catch (Exception e) {
                    e.printStackTrace();
                } finally {
                    if (canvas != null) {
                        surfaceHolder.unlockCanvasAndPost(canvas);
                    }
                }
            }
            timeDiff = System.currentTimeMillis(); // reset time before render/update
        } 
    }
    
    public void start() {
        AndroidGame.running = true;
        thread = new Thread(this);
        thread.start();
    }
    
    public void stop() {
        AndroidGame.running = false;
        while (true) {
            try {
                thread.join();
                break;
            } catch (InterruptedException e) {
                // retry
            }
        }
    }
    
    public void update() {
        if (AndroidGame.state != null) {
            AndroidGame.state.update();
        }
    }
    
    public void render(Canvas canvas) {
        if (AndroidGame.screen != null) {
            AndroidGame.screen.render(canvas);
        }
    }
}
