/*
 * Copyright 2012 David B. Lowery
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.aztechengine.android.touch;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.ArrayBlockingQueue;

import android.view.MotionEvent;
import android.view.View;
import android.view.View.OnTouchListener;

/**
 * <p>This class handles all incoming touch events and stores them in a "touch pipeline". During the update
 * phase of the game loop the {@link #getTouchEvents()} can be called to obtain a Java Collection of all recent
 * touch events.</p>
 * 
 * <p>The handler uses instance pooling to limit the number of orphan TouchEvent objects floating around. This is
 * implemented via a queue (a TouchEvent which is to be freed is added to the end of the queue). The benefit of
 * using instance pooling is that excessive garbage collection of dereferenced TouchEvents could block the main
 * thread causing the application to come to a halt.</p>
 * 
 * @author lowery
 *
 */
public class TouchHandler implements OnTouchListener {
    private static final int POOL_SIZE = 100;
    private static ArrayBlockingQueue<TouchEvent> touchEventPool;
    private static List<TouchEvent> touchEvents;
    private static List<TouchEvent> touchEventsPipeline;

    /**
     * Constructor takes a {@link View} object as an argument and assigns this class to it as a touch
     * listener to intercept all its touch events.
     * 
     * @param view an Android view, specifically a {@link GameSurface}
     * 
     */
    public TouchHandler(View view) {
        view.setOnTouchListener(this);
        
        touchEventPool = new ArrayBlockingQueue<TouchEvent>(POOL_SIZE);
        touchEvents = new ArrayList<TouchEvent>();
        touchEventsPipeline = new ArrayList<TouchEvent>();
        
        for (int i = 0; i < POOL_SIZE; i++) {
            touchEventPool.add(new TouchEvent());
        }
    }
    
    /**
     * When the view to which this {@link OnTouchListener} belongs triggers a MotionEvent, it notifies
     * the listener via this method. The implementation here creates a {@link TouchEvent} and stores it
     * for later processing.
     * 
     * @param view an Android view, specifically a {@link GameSurface}
     * @param event an Android {@link MotionEvent} object
     * 
     */
    public boolean onTouch(View view, MotionEvent event) {
        synchronized (this) {
            int action = event.getActionMasked();
            int pointerIndex = event.getActionIndex();
            int pointerId = event.getPointerId(pointerIndex);

            TouchEvent touchEvent;
            try {
                switch (action) {
                case MotionEvent.ACTION_DOWN:
                case MotionEvent.ACTION_POINTER_DOWN:
                    touchEvent = touchEventPool.take();
                    touchEvent.action = TouchEvent.ACTION_TOUCH_DOWN;
                    touchEvent.pointer = pointerId;
                    touchEvent.x = event.getX(pointerId);
                    touchEvent.y = event.getY(pointerId);
                    touchEventsPipeline.add(touchEvent);
                    break;
                case MotionEvent.ACTION_UP:
                case MotionEvent.ACTION_POINTER_UP:
                case MotionEvent.ACTION_CANCEL:
                    touchEvent = touchEventPool.take();
                    touchEvent.action = TouchEvent.ACTION_TOUCH_UP;
                    touchEvent.pointer = pointerId;
                    touchEvent.x = event.getX(pointerId);
                    touchEvent.y = event.getY(pointerId);
                    touchEventsPipeline.add(touchEvent);
                    break;
                case MotionEvent.ACTION_MOVE:
                    for (int i = 0; i < event.getPointerCount(); i++) {
                        pointerId = event.getPointerId(i);
                        touchEvent = touchEventPool.take();
                        touchEvent.action = TouchEvent.ACTION_TOUCH_MOVE;
                        touchEvent.pointer = pointerId;
                        touchEvent.x = event.getX(pointerId);
                        touchEvent.y = event.getY(pointerId);
                        touchEventsPipeline.add(touchEvent);
                    }
                    break;

                }
            } catch (InterruptedException e) {

            }
        }
        return true;
    }

    /** 
     * Returns a single instance of {@link TouchEvent} to the handler's instance pool.
     * 
     * @param event a touch event
     * 
     */
    public static synchronized void returnToPool(TouchEvent event) {
        try {
            touchEventPool.put(event);
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
    }

    /**
     * Get a Java Collection of the most recently stored touch events and clear the pipeline.
     * 
     * @return a list of touch events
     * 
     */
    public static synchronized List<TouchEvent> getTouchEvents() {
        for (int i = 0; i < touchEvents.size(); i++)
            returnToPool(touchEvents.get(i));
        touchEvents.clear();
        touchEvents.addAll(touchEventsPipeline);
        touchEventsPipeline.clear();
        return touchEvents;
    }
}

